@@ -1,5 +1,6 @@
 package Color::Calc;
 
+use attributes;
 use strict;
 use utf8;
 use warnings;
@@ -9,26 +10,55 @@ use Exporter;
 use Params::Validate qw(:all);
 use POSIX;
 
-use Graphics::ColorNames qw( hex2tuple tuple2hex );
-use Graphics::ColorNames::HTML;
+use Scalar::Util qw(dualvar);
+use List::Util qw(min max reduce sum);
 
-our $VERSION = '1.052';
+use Graphics::ColorNames qw( hex2tuple tuple2hex );
+use Graphics::ColorNames::HTML qw();
+
+our $VERSION = "1.070";
 $VERSION = eval $VERSION;
 
 our $MODE = ();
 
 my %__HTMLColors = ();
-our @__subs = qw( blend blend_bw bw contrast contrast_bw dark get grey gray invert light mix );
+our @__subs = qw(
+  blend blend_bw
+  bw
+  contrast contrast_bw
+  dark
+  get
+  gray
+  grey
+  invert
+  light
+  mix
+  opposite
+  round
+  safe
+);
+
+sub __put_tuple { map { my $a = int($_); length($a) % 3 ? $a : dualvar($a, "0$a") } @_ };
+sub __put_hex 	{ dualvar((reduce { ($a << 8) | ($b & 0xFF) } @_), tuple2hex(@_)) };
+sub __put_html	{ my $col = lc(tuple2hex(@_)); $__HTMLColors{$col} || '#'.$col; };
+sub __put_object{ return Graphics::ColorObject->new_RGB255( \@_, '', '' ); };
+sub __put_obj	{ return Color::Object->newRGB(map { 255*$_; } @_); };
 
 my %__formats = (
-  'tuple'	=> sub { return __normtuple_out(@_); },
-  'hex'		=> sub { return tuple2hex((@_)); },
-  'html'	=> sub { my $col = lc(tuple2hex((@_))); return $__HTMLColors{$col} || '#'.$col; },
-  'object'	=> sub { eval { require Graphics::ColorObject; }; return Graphics::ColorObject->new_RGB255( [(@_)] ); },
-  'obj'		=> sub { eval { require Color::Object; }; return Color::Object->newRGB(map { 255*$_; } @_); },
-  'pdf'		=> sub { return '#'.(tuple2hex(@_)) },
+  map { m/^__put_(.*)/ ? ( $1 => $Color::Calc::{$_} ) : () }
+    keys %Color::Calc::
 );
 
+# use Data::Dumper;
+# print STDERR Dumper(\%__formats);
+
+my %__formats_require = (
+  'obj'		=> 'Color::Object',
+  'object'	=> 'Graphics::ColorObject',
+);
+
+$__formats{'pdf'} = $__formats{'html'};
+
 my @__formats = keys %__formats;
 my $__formats_re = join('|', @__formats,'__MODEvar');
 
@@ -44,7 +74,7 @@ our @EXPORT = ('color', map({"color_$_"} @__formats, map({my $s=$_; (map{$s.'_'.
 our @ISA = ('Exporter');
 
 my %new_param = (
-  'ColorScheme' => { type => SCALAR, optional => 1 },
+  'ColorScheme' => { type => SCALAR | HANDLE | HASHREF | ARRAYREF | CODEREF, optional => 1 },
   'OutputFormat' => { type => SCALAR, untaint => 1, regexp => qr($__formats_re), optional => 1 },
 );
 
@@ -52,14 +82,25 @@ sub new {
   my $pkg = shift; validate(@_, \%new_param);
   my $self = {@_}; bless($self, $pkg);
 
-  if(!ref($self->{'ColorScheme'})) {
+  unless(UNIVERSAL::isa($self->{'ColorScheme'}, 'Graphics::ColorNames')) {
     my %ColorNames;
-    tie %ColorNames, 'Graphics::ColorNames', ($self->{'ColorScheme'} || 'X');
+    if(defined $self->{'ColorScheme'}) {
+      if(!ref $self->{'ColorScheme'} && $self->{'ColorScheme'} =~ m/^([[:alnum:]_]+)$/) {
+        my $module = 'Graphics::ColorNames::'.$1;
+        eval "use $module;"; croak $! if $!;
+        my $names = UNIVERSAL::can($module, 'NamesRgbTable');
+        croak "$module is not compatible with Graphics::ColorNames" if !$names;
+        $self->{'ColorScheme'} = &$names();
+      }
+
+      tie %ColorNames, 'Graphics::ColorNames', $self->{'ColorScheme'};
+    } else {
+      tie %ColorNames, 'Graphics::ColorNames';
+    }
     $self->{'ColorScheme'} = \%ColorNames;
   }
 
   $self->set_output_format($self->{'OutputFormat'} || 'tuple');
-
   return $self;
 }
 
@@ -69,10 +110,10 @@ sub __get_default {
   return $__default_object;
 }
 
-my $__tuple_object = undef;
-sub __get_tuple {
-  $__tuple_object = __PACKAGE__->new('OutputFormat' => 'tuple') unless $__tuple_object;
-  return $__tuple_object;
+my $__raw_object = undef;
+sub __get_raw {
+  $__raw_object = __PACKAGE__->new('OutputFormat' => 'tuple') unless $__raw_object;
+  return $__raw_object;
 }
 
 my %import_param = ( 
@@ -126,17 +167,14 @@ sub __import {
   return 1;
 }
 
+sub __dualvar_tuple {
+  my $str = shift;
+  my $num = reduce { ($a << 8) | ($b & 0xFF) } @_;
+  return dualvar $num, $str;
+}
 
 sub __normtuple_in {
-  return map { ($_ < 0) ? 0 : (($_ > 255) ? 255 : int($_+.5)) } @_;
-}
-
-sub __normtuple_out {
-  return map { (length $_) == 3 ? '0'.$_ : $_ } __normtuple_in(@_)
-}
-
-sub __normtuple {
-  return __normtuple_out(__normtuple_in(@_));
+  return map { (!defined($_) || $_ < 0) ? 0 : (($_ > 255) ? 255 : int($_+.5)) } @_;
 }
 
 sub __is_col_val {
@@ -186,7 +224,7 @@ sub __get {
   }
   else {
     my $col = $self->{'ColorScheme'}->{$$p[0]};
-    if($col) {
+    if(defined $col) {
       shift @$p; 
       return hex2tuple($col);
     } else {
@@ -196,12 +234,23 @@ sub __get {
   }
 }
 
+sub __require_format {
+  my $new_fmt = shift;
+
+  if(exists $__formats_require{$new_fmt}) {
+    eval "use $__formats_require{$new_fmt}()"; 
+    croak $@ if $@;
+  }
+  return 1;
+}
+
 sub set_output_format {
   validate_pos(@_, { isa => __PACKAGE__ }, { type => SCALAR, regexp => qr($__formats_re) });
   my $self = shift;
+  my $new_fmt = shift;
 
   my $old = $self->{'OutputFormat'};
-  $self->{'OutputFormat'} = shift;
+  $self->{'OutputFormat'} = $new_fmt;
 
   $self->{'__put'} = $self->{'OutputFormat'} eq '__MODEvar' 
     ? sub{ return $__formats{$MODE || 'tuple'}->(@_); }
@@ -223,85 +272,6 @@ sub __get_self {
   }
 }
 
-sub get { 
-  my $self = __get_self(\@_); 
-  return $self->__put($self->__get(\@_)); 
-}
-
-sub invert {
-  my $self = __get_self(\@_);
-  return $self->__put(map { 255 - $_ } $self->__get(\@_));
-}
-
-sub bw {
-  my $self = __get_self(\@_);
-  my @c = $self->__get(\@_);
-  my $g = $c[0]*.3 + $c[1]*.59 + $c[2]*.11;
-  return $self->__put(__normtuple($g,$g,$g));
-}
-
-*grey = \&bw;
-*gray = \&bw;
-
-sub mix {
-  my $self = __get_self(\@_);
-  my @c1 = ($self->__get(\@_,1));
-  my @c2 = ($self->__get(\@_));
-  my $alpha = shift(@_); $alpha = 0.5 unless defined $alpha;
-
-  return $self->__put(__normtuple(
-    ($c1[0] + ($c2[0]-$c1[0])*$alpha),
-    ($c1[1] + ($c2[1]-$c1[1])*$alpha),
-    ($c1[2] + ($c2[2]-$c1[2])*$alpha) ));
-}
-
-sub light {
-  my $self = __get_self(\@_);
-  return $self->__put(__get_tuple->mix([$self->__get(\@_)],[255,255,255],shift));
-}
-
-sub dark {
-  my $self = __get_self(\@_);
-  return $self->__put(__get_tuple->mix([$self->__get(\@_)],[0,0,0],shift));
-}
-
-sub contrast {
-  my $self = __get_self(\@_);
-  my @rgb = $self->__get(\@_);
-  my $cut = (shift || .5) * 255;
-  return $self->__put(map { $_ >= $cut ? 0 : '0255' } @rgb);
-}
-
-sub contrast_bw {
-  my $self = __get_self(\@_);
-  my @rgb = $self->__get(\@_);
-  return $self->__put(__get_tuple->contrast([__get_tuple->bw(@rgb)], shift));
-}
-
-sub blend {
-  my $self = __get_self(\@_);
-  my @c1 = $self->__get(\@_);
-  return $self->mix(\@c1,[__get_tuple->contrast(\@c1)],shift);
-}
-
-sub blend_bw {
-  my $self = __get_self(\@_);
-  my @c = $self->__get(\@_);
-  return $self->mix(\@c,[__get_tuple->contrast_bw(\@c)],shift);
-}
-
-foreach my $format (@__formats) {
-  __import(__PACKAGE__, 'Prefix' => 'color', '__Suffix' => "_$format", 'OutputFormat' => $format);
-  __import(__PACKAGE__, 'Prefix' => '',      '__Suffix' => "_$format", 'OutputFormat' => $format);
-}
-
-__import(__PACKAGE__, 'Prefix' => 'color', 'OutputFormat' => '__MODEvar');
-
-1;
-__END__
-
-=encoding utf8
-
 =head1 NAME
 
 Color::Calc - Simple calculations with RGB colors.
@@ -326,7 +296,7 @@ can be used to create a full color scheme from a few colors.
 
 =over
 
-=item Color::Calc->new( … )
+=item Color::Calc->new( ... )
 
 This class method creates a new C<Color::Calc> object.
 
@@ -347,6 +317,10 @@ names defined in the HTML 4.0 specification). For a full list of
 possible values, please refer to the documentation of of
 C<Graphics::ColorNames>.
 
+Unlike C<Graphics::ColorNames>, barewords are I<always> interpreted as a module
+name under C<Graphics::ColorNames>. If you really want to use a filename like
+"foo", you have to write it as "./foo".
+
 Default: C<X> (Note: This is incompatible with HTML color names).
 
 =item OutputFormat
@@ -362,7 +336,8 @@ guaranteed to have a C<length> that is not a multiple of three.
 
 =item hex
 
-Returns a hexadecimal RGB value as a string in the format RRGGBB.
+Returns a hexadecimal RGB value as a scalar that contains a string in the
+format RRGGBB and a number representing the hexadecimal number 0xRRGGBB.
 
 =item html
 
@@ -371,8 +346,8 @@ i.e. I<#RRGGBB> or one of the sixteen HTML color names.
 
 =item obj
 	
-Returns a C<Color::Object> reference. The module
-C<Color::Object> must be installed, of course.
+(DEPRECATED) Returns a C<Color::Object> reference. The module C<Color::Object>
+must be installed, of course.
 
 =item object
 
@@ -397,7 +372,7 @@ Default: C<__MODEvar> (for compatibility)
 
 =back
 
-=item Color::Calc->import( … )
+=item Color::Calc->import( ... )
 
 This method creates a new, hidden object and binds its methods to the namespace
 of the calling module.
@@ -501,10 +476,43 @@ reference (here: C<$cc>) or through the method names imported by C<import>
 Returns C<$color> as-is (but in the selected output format). This
 function can be used for color format conversion/normalisation.
 
+=cut
+
+sub get { 
+  my $self = __get_self(\@_); 
+  return $self->__put($self->__get(\@_)); 
+}
+
 =item $cc->invert($color) / color_invert($color)
 
 Returns the inverse of C<$color>.
 
+=cut
+
+sub invert {
+  my $self = __get_self(\@_);
+  return $self->__put(map { 255 - $_ } $self->__get(\@_));
+}
+
+=item $cc->opposite($color) / color_opposite($color)
+
+Returns a color that is on the opposite side of the color wheel but roughly
+keeps the saturation and lightness.
+
+=cut 
+
+sub opposite {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+
+  my $min = min @rgb;
+  my $max = max @rgb;
+
+  return $self->__put(
+    map { $max - $_ + $min } @rgb
+  );
+}
+
 =item $cc->bw($color) / color_bw($color)
 
 =item $cc->grey($color) / color_grey($color)
@@ -513,6 +521,74 @@ Returns the inverse of C<$color>.
 
 Converts C<$color> to greyscale.
 
+=cut
+
+sub bw {
+  my $self = __get_self(\@_);
+  my @c = $self->__get(\@_);
+  my $g = $c[0]*.3 + $c[1]*.59 + $c[2]*.11;
+  return $self->__put($g, $g, $g);
+}
+
+*grey = \&bw;
+*gray = \&bw;
+
+=item $cc->round($color, $value_count) / color_round($color, $value_count)
+
+Rounds each component to to the nearest number determined by dividing the range
+0..255 into C<$value_count>+1 portions.
+
+The default for C<$value_count> is 6, yielding S<6^3 = 216> colors.  Values
+that are one higher than divisors of 255 yield the best results (e.g. 3+1, 5+1,
+7+1, 9+1, 15+1, 17+1, ...).
+
+=cut 
+
+sub round {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+  my $steps = shift || 6;
+  $steps--;
+
+  return $self->__put(
+    map { int(int( $_ * $steps / 255 + 0.5) * 255 / $steps + 0.5) } @rgb
+  );
+}
+
+=item $cc->safe($color) / color_safe($color)
+
+Rounds each color component to a multiple of 0x33 (dec. 51) or to a named color
+defined in the HTML 4.01 specification.
+
+Historically, these colors have been known as web-safe colors. They still
+provide a convenient color palette.
+
+=cut
+
+sub __dist2 {
+  my @a = splice @_, 0, 3;
+  return sum map { POSIX::pow($_ - shift @a, 2) } @_;
+}
+
+sub safe {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+
+  my @new_rgb = __get_raw->round(@rgb);
+  my $new_d2 = __dist2(@rgb, @new_rgb);
+
+  foreach my $h (keys %__HTMLColors) {
+    my @h_rgb = hex2tuple($h);
+    my $h_d2 = __dist2(@rgb, @h_rgb);
+
+    if($h_d2 <= $new_d2) {
+      @new_rgb = @h_rgb;
+      $new_d2 = $h_d2;
+    }
+  }
+  return $self->__put(@new_rgb);
+}
+
 =item $cc->mix($color1, $color2 [, $alpha]) / color_mix($color1, $color2 [, $alpha])
 
 Returns a color that is the mixture of C<$color1> and C<$color2>.
@@ -520,6 +596,20 @@ Returns a color that is the mixture of C<$color1> and C<$color2>.
 The optional C<$alpha> parameter can be a value between 0.0 (use
 C<$color1> only) and 1.0 (use C<$color2> only), the default is 0.5.
 
+=cut
+
+sub mix {
+  my $self = __get_self(\@_);
+  my @c1 = ($self->__get(\@_,1));
+  my @c2 = ($self->__get(\@_));
+  my $alpha = shift(@_); $alpha = 0.5 unless defined $alpha;
+
+  return $self->__put(
+    ($c1[0] + ($c2[0]-$c1[0])*$alpha),
+    ($c1[1] + ($c2[1]-$c1[1])*$alpha),
+    ($c1[2] + ($c2[2]-$c1[2])*$alpha) );
+}
+
 =item $cc->light($color [, $alpha]) / color_light($color [, $alpha])
 
 Returns a lighter version of C<$color>, i.e. returns
@@ -528,6 +618,13 @@ C<mix($color,[255,255,255],$alpha)>.
 The optional C<$alpha> parameter can be a value between 0.0 (use C<$color>
 only) and 1.0 (use [255,255,255] only), the default is 0.5.
 
+=cut
+
+sub light {
+  my $self = __get_self(\@_);
+  return $self->__put(__get_raw->mix([$self->__get(\@_)],[255,255,255],shift));
+}
+
 =item $cc->dark($color [, $alpha]) / color_dark($color [, $alpha])
 
 Returns a darker version of C<$color>, i.e. returns
@@ -536,6 +633,14 @@ C<mix($color,[0,0,0],$alpha)>.
 The optional C<$alpha> parameter can be a value between 0.0 (use
 C<$color> only) and 1.0 (use [0,0,0] only), the default is 0.5.
 
+=cut
+
+sub dark {
+  my $self = __get_self(\@_);
+  return $self->__put(__get_raw->mix([$self->__get(\@_)],[0,0,0],shift));
+}
+
+
 =item $cc->contrast($color [, $cut]) / color_contrast($color [, $cut])
 
 Returns a color that has the highest possible contrast to the input
@@ -547,6 +652,15 @@ to 255 otherwise.
 
 The default for C<$cut> is .5, representing a cutoff between 127 and 128.
 
+=cut
+
+sub contrast {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+  my $cut = (shift || .5) * 255;
+  return $self->__put(map { $_ >= $cut ? 0 : 255 } @rgb);
+}
+
 =item $cc->contrast_bw($color [, $cut]) / color_contrast_bw($color [, $cut])
 
 Returns black or white, whichever has the higher contrast to C<$color>.
@@ -556,8 +670,15 @@ above C<($cut * 255)> and white otherwise.
 
 The default for C<$cut> is .5, representing a cutoff between 127 and 128.
 
-=item $cc->blend($color [, $alpha])
-color_blend($color [, $alpha])
+=cut
+
+sub contrast_bw {
+  my $self = __get_self(\@_);
+  my @rgb = $self->__get(\@_);
+  return $self->__put(__get_raw->contrast([__get_raw->bw(@rgb)], shift));
+}
+
+=item $cc->blend($color [, $alpha]) / color_blend($color [, $alpha])
 
 Returns a color that blends into the background, i.e. it returns
 C<mix($color,contrast($color),$alpha)>.
@@ -573,6 +694,14 @@ them returns a color somewhere between them.
 You might want to use C<mix($color, $background, $alpha)> instead
 if you know the real background color.
 
+=cut
+
+sub blend {
+  my $self = __get_self(\@_);
+  my @c1 = $self->__get(\@_);
+  return $self->mix(\@c1,[__get_raw->contrast(\@c1)],shift);
+}
+
 =item $cc->blend_bw($color [, $alpha]) / color_blend_bw($color [, $alpha])
 
 Returns a mix of C<$color> and black or white, whichever has the
@@ -581,6 +710,14 @@ higher contrast to C<$color>.
 The optional C<$alpha> parameter can be a value between 0.0 (use
 C<$color> only) and 1.0 (use black/white only), the default is 0.5.
 
+=cut
+
+sub blend_bw {
+  my $self = __get_self(\@_);
+  my @c = $self->__get(\@_);
+  return $self->mix(\@c,[__get_raw->contrast_bw(\@c)],shift);
+}
+
 =back
 
 =head3 Functions
@@ -597,15 +734,15 @@ these functions are exported.
 
 =over
 
-=item color, color_mix, …
+=item color, color_mix, ...
 
 Use C<$Color::Calc::MODE> as the output format. This is the default.
 
-=item color_hex, color_mix_html, …
+=item color_hex, color_mix_html, ...
 
 Use C<hex> as the output format.
 
-=item color_html, color_mix_html, …
+=item color_html, color_mix_html, ...
 
 Use C<html> as the output format. Please note that the color names recognized
 are still based on X's color names, which are incompatible with HTML. You can't
@@ -614,29 +751,42 @@ use the output of these functions as input for other color_*_html functions.
 See L<Color::Calc::WWW> for an alternative that does not suffer from this
 problem.
 
-=item color_pdf, color_mix_pdf, …
+=item color_pdf, color_mix_pdf, ...
 
 Use C<pdf> as the output format.
 
-=item color_object, color_mix_object, …
+=item color_object, color_mix_object, ...
 
 Use C<object> as the output format.
 
 =back
 
+=cut
+
+foreach my $format (@__formats) {
+  next if !eval{__require_format($format)};
+  __import(__PACKAGE__, 'Prefix' => 'color', '__Suffix' => "_$format", 'OutputFormat' => $format);
+  __import(__PACKAGE__, 'Prefix' => '',      '__Suffix' => "_$format", 'OutputFormat' => $format);
+}
+
+__import(__PACKAGE__, 'Prefix' => 'color', 'OutputFormat' => '__MODEvar');
+
 =head1 SEE ALSO
 
 L<Graphics::ColorNames> (required); L<Graphics::ColorObject> (optional)
 
 =head1 AUTHOR
 
-Claus Färber <CFAERBER@cpan.org>
+Claus FE<auml>rber <CFAERBER@cpan.org>
 
 =head1 LICENSE
 
-Copyright © 2004-2009 Claus Färber. All rights reserved.
+Copyright 2004-2010 Claus FE<auml>rber. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify it under
 the same terms as Perl itself.
 
 =cut
+
+1;
+__END__
